"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveMinify = exports.resolveBuiltinsOptions = void 0;
const browserslist_1 = require("browserslist");
function resolveDefine(define) {
    const entries = Object.entries(define).map(([key, value]) => [
        key,
        value === undefined ? "undefined" : value
    ]);
    return Object.fromEntries(entries);
}
function resolveHtml(html) {
    return html.map(c => {
        for (const key in c.meta) {
            const value = c.meta[key];
            if (typeof value === "string") {
                c.meta[key] = {
                    name: key,
                    content: value
                };
            }
        }
        return c;
    });
}
function resolveDecorator(decorator) {
    if (decorator === false) {
        return undefined;
    }
    if (decorator === undefined || decorator === true) {
        decorator = {};
    }
    return Object.assign({
        legacy: true,
        emitMetadata: true,
        useDefineForClassFields: true
    }, decorator);
}
function resolveBuiltinsOptions(builtins, { contextPath, isProduction }) {
    const browserslist = (0, browserslist_1.loadConfig)({ path: contextPath }) || [];
    return {
        ...builtins,
        define: resolveDefine(builtins.define || {}),
        html: resolveHtml(builtins.html || []),
        browserslist,
        decorator: resolveDecorator(builtins.decorator),
        minify: resolveMinify(builtins, isProduction)
    };
}
exports.resolveBuiltinsOptions = resolveBuiltinsOptions;
function resolveMinify(builtins, isProduction) {
    if (builtins.minify) {
        if (builtins.minify === true) {
            return {
                enable: true,
                passes: 1
            };
        }
        else {
            return {
                ...builtins.minify,
                enable: true
            };
        }
    }
    else if (builtins.minify === false) {
        return {
            enable: false,
            passes: 1
        };
    }
    else {
        return {
            enable: isProduction,
            passes: 1
        };
    }
}
exports.resolveMinify = resolveMinify;
//# sourceMappingURL=builtins.js.map