"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyRspackOptionsDefaults = exports.applyRspackOptionsBaseDefaults = void 0;
const path_1 = __importDefault(require("path"));
const target_1 = require("./target");
const NODE_MODULES_REGEXP = /[\\/]node_modules[\\/]/i;
const D = (obj, prop, value) => {
    if (obj[prop] === undefined) {
        obj[prop] = value;
    }
};
const F = (obj, prop, factory) => {
    if (obj[prop] === undefined) {
        obj[prop] = factory();
    }
};
const A = (obj, prop, factory) => {
    const value = obj[prop];
    if (value === undefined) {
        obj[prop] = factory();
    }
    else if (Array.isArray(value)) {
        let newArray = undefined;
        for (let i = 0; i < value.length; i++) {
            const item = value[i];
            if (item === "...") {
                if (newArray === undefined) {
                    newArray = value.slice(0, i);
                    obj[prop] = newArray;
                }
                const items = factory();
                if (items !== undefined) {
                    for (const item of items) {
                        newArray.push(item);
                    }
                }
            }
            else if (newArray !== undefined) {
                newArray.push(item);
            }
        }
    }
};
function applyRspackOptionsBaseDefaults(options) {
    F(options, "context", () => process.cwd());
    applyInfrastructureLoggingDefaults(options.infrastructureLogging);
}
exports.applyRspackOptionsBaseDefaults = applyRspackOptionsBaseDefaults;
/**
 * @param {InfrastructureLogging} infrastructureLogging options
 * @returns {void}
 */
const applyInfrastructureLoggingDefaults = infrastructureLogging => {
    F(infrastructureLogging, "stream", () => process.stderr);
    const tty = 
    /** @type {any} */ infrastructureLogging.stream.isTTY &&
        process.env.TERM !== "dumb";
    D(infrastructureLogging, "level", "info");
    D(infrastructureLogging, "debug", false);
    D(infrastructureLogging, "colors", tty);
    D(infrastructureLogging, "appendOnly", !tty);
};
const applyOutputDefaults = (output, context) => {
    D(output, "hashFunction", "xxhash64");
    F(output, "path", () => path_1.default.resolve(context, "dist"));
    D(output, "publicPath", "auto");
    return output;
};
const applyOptimizationDefaults = (optimization, { production, development, css, records }) => {
    D(optimization, "removeAvailableModules", false);
    D(optimization, "removeEmptyChunks", true);
    D(optimization, "mergeDuplicateChunks", true);
    D(optimization, "flagIncludedChunks", production);
    F(optimization, "moduleIds", () => {
        if (production)
            return "deterministic";
        if (development)
            return "named";
        return "natural";
    });
    F(optimization, "chunkIds", () => {
        if (production)
            return "deterministic";
        if (development)
            return "named";
        return "natural";
    });
    F(optimization, "sideEffects", () => (production ? true : "flag"));
    D(optimization, "providedExports", true);
    D(optimization, "usedExports", production);
    D(optimization, "innerGraph", production);
    D(optimization, "mangleExports", production);
    D(optimization, "concatenateModules", production);
    D(optimization, "runtimeChunk", false);
    D(optimization, "emitOnErrors", !production);
    D(optimization, "checkWasmTypes", production);
    D(optimization, "mangleWasmImports", false);
    D(optimization, "portableRecords", records);
    D(optimization, "realContentHash", production);
    D(optimization, "minimize", production);
    F(optimization, "nodeEnv", () => {
        if (production)
            return "production";
        if (development)
            return "development";
        return false;
    });
    const { splitChunks } = optimization;
    if (splitChunks) {
        A(splitChunks, "defaultSizeTypes", () => css ? ["javascript", "css", "unknown"] : ["javascript", "unknown"]);
        D(splitChunks, "hidePathInfo", production);
        D(splitChunks, "chunks", "async");
        D(splitChunks, "usedExports", optimization.usedExports === true);
        D(splitChunks, "minChunks", 1);
        F(splitChunks, "minSize", () => (production ? 20000 : 10000));
        F(splitChunks, "minRemainingSize", () => (development ? 0 : undefined));
        F(splitChunks, "enforceSizeThreshold", () => (production ? 50000 : 30000));
        F(splitChunks, "maxAsyncRequests", () => (production ? 30 : Infinity));
        F(splitChunks, "maxInitialRequests", () => (production ? 30 : Infinity));
        D(splitChunks, "automaticNameDelimiter", "-");
        const { cacheGroups } = splitChunks;
        F(cacheGroups, "default", () => ({
            idHint: "",
            reuseExistingChunk: true,
            minChunks: 2,
            priority: -20
        }));
        F(cacheGroups, "defaultVendors", () => ({
            idHint: "vendors",
            reuseExistingChunk: true,
            test: NODE_MODULES_REGEXP,
            priority: -10
        }));
    }
};
function applyRspackOptionsDefaults(options) {
    F(options, "context", () => process.cwd());
    /** @todo  */
    F(options, "target", () => {
        return (0, target_1.getDefaultTarget)(options.context);
    });
    const { mode, context } = options;
    const development = mode === "development";
    const production = mode === "production" || !mode;
    F(options, "devtool", () => (development ? "eval-cheap-source-map" : ""));
    applyOutputDefaults(options.output, context);
    applyOptimizationDefaults(options.optimization, {
        development,
        production,
        css: false,
        records: false
    });
}
exports.applyRspackOptionsDefaults = applyRspackOptionsDefaults;
//# sourceMappingURL=defaults.js.map