"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveEntryOptions = void 0;
const path_1 = __importDefault(require("path"));
function resolveEntryOptions(options, context) {
    if (typeof options === "undefined" || options === null) {
        return {
            main: normalizeEntryItem("main", {
                import: [path_1.default.resolve(context.context, "src", "index.js")]
            })
        };
    }
    else if (typeof options === "string") {
        return {
            main: normalizeEntryItem("main", {
                import: [options]
            })
        };
    }
    else if (Array.isArray(options)) {
        return {
            main: normalizeEntryItem("main", {
                import: [...options]
            })
        };
    }
    else if (typeof options === "object") {
        return Object.fromEntries(Object.entries({ ...options }).map(([key, value]) => {
            if (Array.isArray(value)) {
                return [
                    key,
                    normalizeEntryItem(key, {
                        import: [...value]
                    })
                ];
            }
            else if (typeof value === "object") {
                return [key, normalizeEntryItem(key, value)];
            }
            else {
                return [
                    key,
                    normalizeEntryItem(key, {
                        import: [value]
                    })
                ];
            }
        }));
    }
    else {
        return {};
    }
}
exports.resolveEntryOptions = resolveEntryOptions;
function normalizeEntryItem(entryName, entryItem) {
    return {
        import: Array.isArray(entryItem.import)
            ? entryItem.import
            : [entryItem.import],
        runtime: entryItem.runtime === false ? undefined : entryItem.runtime
    };
}
//# sourceMappingURL=entry.js.map