/// <reference types="node" />
import type { RawModuleRuleUse, RawModuleRule, RawModuleOptions, JsAssetInfo } from "@rspack/binding";
import { ResolveRequest } from "enhanced-resolve";
import { Compiler } from "../compiler";
import { Logger } from "../logging/Logger";
import { ResolveOptions } from "../ResolverFactory";
import Hash from "../util/hash";
import { ResolvedContext } from "./context";
import { ResolvedDevtool } from "./devtool";
import { ResolvedMode } from "./mode";
import { Resolve } from "./resolve";
import { ResolvedTarget } from "./target";
export declare type Condition = string | RegExp;
export interface ModuleRule {
    name?: string;
    test?: Condition;
    include?: Condition | Condition[];
    exclude?: Condition | Condition[];
    resource?: Condition;
    resourceQuery?: Condition;
    use?: ModuleRuleUse[];
    type?: RawModuleRule["type"];
    resolve?: Resolve;
}
export interface Module {
    rules?: ModuleRule[];
    parser?: RawModuleOptions["parser"];
}
interface ResolvedModuleRule {
    test?: RawModuleRule["test"];
    include?: RawModuleRule["include"];
    exclude?: RawModuleRule["exclude"];
    resource?: RawModuleRule["resource"];
    resourceQuery?: RawModuleRule["resourceQuery"];
    use?: RawModuleRuleUse[];
    type?: RawModuleRule["type"];
}
export interface ResolvedModule {
    rules: ResolvedModuleRule[];
    parser?: RawModuleOptions["parser"];
}
interface LoaderContextInternal {
    source: number[];
    sourceMap: string | undefined | null;
    additionalData: AdditionalData | undefined | null;
    resource: string;
    resourcePath: string;
    resourceQuery: string | null;
    resourceFragment: string | null;
    cacheable: boolean;
    buildDependencies: string[];
}
export interface LoaderContext extends Pick<LoaderContextInternal, "resource" | "resourcePath" | "resourceQuery" | "resourceFragment"> {
    version: 2;
    async(): (err: Error | null, content: string | Buffer, sourceMap?: string | SourceMap, additionalData?: AdditionalData) => void;
    callback(err: Error | null, content: string | Buffer, sourceMap?: string | SourceMap, additionalData?: AdditionalData): void;
    cacheable(cacheable: boolean): void;
    sourceMap: boolean;
    rootContext: string;
    context: string;
    loaderIndex: number;
    mode: ResolvedMode;
    hot?: boolean;
    getOptions(schema?: any): unknown;
    resolve(context: string, request: string, callback: (arg0: null | Error, arg1?: string | false, arg2?: ResolveRequest) => void): void;
    getResolve(options: ResolveOptions): (context: any, request: any, callback: any) => Promise<any>;
    getLogger(name: string): Logger;
    emitError(error: Error): void;
    emitWarning(warning: Error): void;
    emitFile(name: string, content: string | Buffer, sourceMap?: string, assetInfo?: JsAssetInfo): void;
    addDependency(file: string): void;
    addContextDependency(context: string): void;
    addMissingDependency(missing: string): void;
    clearDependencies(): void;
    getDependencies(): string[];
    getContextDependencies(): string[];
    getMissingDependencies(): string[];
    addBuildDependency(file: string): void;
    fs: any;
    utils: {
        absolutify: (context: string, request: string) => string;
        contextify: (context: string, request: string) => string;
        createHash: (algorithm?: string) => Hash;
    };
    query: unknown;
    data: unknown;
}
export declare type GetCompiler = () => Compiler;
export interface ComposeJsUseOptions {
    devtool: ResolvedDevtool;
    context: ResolvedContext;
    target: ResolvedTarget;
    getCompiler: GetCompiler;
}
export interface SourceMap {
    version: number;
    sources: string[];
    mappings: string;
    file?: string;
    sourceRoot?: string;
    sourcesContent?: string[];
    names?: string[];
}
export interface AdditionalData {
    [index: string]: any;
}
export interface LoaderResult {
    cacheable: boolean;
    buildDependencies?: string[];
    content: string | Buffer;
    sourceMap?: string | SourceMap;
    additionalData?: AdditionalData;
    fileDependencies: string[];
    contextDependencies: string[];
    missingDependencies: string[];
}
export interface Loader {
    (this: LoaderContext, content: string | Buffer, sourceMap?: string | SourceMap, additionalData?: AdditionalData): void;
    displayName?: string;
    raw?: boolean;
}
declare type BuiltinLoader = string;
declare type ModuleRuleUse = {
    builtinLoader: BuiltinLoader;
    options?: unknown;
    name?: string;
} | {
    loader: Loader | string;
    options?: unknown;
    name?: string;
    query?: string;
    data?: unknown;
};
export declare function createRawModuleRuleUses(uses: ModuleRuleUse[], options: ComposeJsUseOptions): RawModuleRuleUse[];
export declare function resolveModuleOptions(module: Module, options: ComposeJsUseOptions): ResolvedModule;
export {};
//# sourceMappingURL=module.d.ts.map