"use strict";
/*
    MIT License http://www.opensource.org/licenses/mit-license.php
    Author Tobias Koppers @sokra
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.hooks = exports.configureDefaultLogger = exports.getLogger = void 0;
const SyncBailHook = require("tapable/lib/SyncBailHook");
const { Logger } = require("./Logger");
const createConsoleLogger = require("./createConsoleLogger");
/** @type {createConsoleLogger.LoggerOptions} */
let currentDefaultLoggerOptions = {
    level: "info",
    debug: false,
    console
};
let currentDefaultLogger = createConsoleLogger(currentDefaultLoggerOptions);
/**
 * @param {string} name name of the logger
 * @returns {Logger} a logger
 */
const getLogger = name => {
    return new Logger((type, args) => {
        if (exports.hooks.log.call(name, type, args) === undefined) {
            currentDefaultLogger(name, type, args);
        }
    }, childName => exports.getLogger(`${name}/${childName}`));
};
exports.getLogger = getLogger;
/**
 * @param {createConsoleLogger.LoggerOptions} options new options, merge with old options
 * @returns {void}
 */
const configureDefaultLogger = options => {
    Object.assign(currentDefaultLoggerOptions, options);
    currentDefaultLogger = createConsoleLogger(currentDefaultLoggerOptions);
};
exports.configureDefaultLogger = configureDefaultLogger;
exports.hooks = {
    log: new SyncBailHook(["origin", "type", "args"])
};
//# sourceMappingURL=runtime.js.map