"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RspackOptionsApply = void 0;
const graceful_fs_1 = __importDefault(require("graceful-fs"));
const NodeTargetPlugin_1 = require("./node/NodeTargetPlugin");
const ResolveSwcPlugin_1 = require("./web/ResolveSwcPlugin");
const cleverMerge_1 = require("./util/cleverMerge");
class RspackOptionsApply {
    constructor() { }
    process(options, compiler) {
        compiler.outputPath = options.output.path;
        compiler.name = options.name;
        compiler.outputFileSystem = graceful_fs_1.default;
        if (compiler.options.target.includes("node")) {
            new NodeTargetPlugin_1.NodeTargetPlugin().apply(compiler);
        }
        // after we migrate minify to minimze, we could remove it
        if (options.optimization.minimize || options.builtins.minify) {
            if (options.optimization.minimizer) {
                for (const minimizer of options.optimization.minimizer) {
                    if (minimizer !== "...") {
                        minimizer.apply(compiler);
                    }
                }
            }
        }
        new ResolveSwcPlugin_1.ResolveSwcPlugin().apply(compiler);
        if (!compiler.inputFileSystem) {
            throw new Error("No input filesystem provided");
        }
        compiler.resolverFactory.hooks.resolveOptions
            .for("normal")
            .tap("RspackOptionsApply", resolveOptions => {
            resolveOptions = (0, cleverMerge_1.cleverMerge)(options.resolve, resolveOptions);
            resolveOptions.fileSystem = compiler.inputFileSystem;
            return resolveOptions;
        });
        compiler.resolverFactory.hooks.resolveOptions
            .for("context")
            .tap("RspackOptionsApply", resolveOptions => {
            resolveOptions = (0, cleverMerge_1.cleverMerge)(options.resolve, resolveOptions);
            resolveOptions.fileSystem = compiler.inputFileSystem;
            resolveOptions.resolveToContext = true;
            return resolveOptions;
        });
    }
}
exports.RspackOptionsApply = RspackOptionsApply;
//# sourceMappingURL=rspackOptionsApply.js.map