"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRawFromSource = exports.createSourceFromRaw = void 0;
const webpack_sources_1 = require("webpack-sources");
function createSourceFromRaw(source) {
    if (source.isRaw) {
        return new webpack_sources_1.RawSource(
        // @ts-expect-error: webpack-sources can accept buffer as source, see: https://github.com/webpack/webpack-sources/blob/9f98066311d53a153fdc7c633422a1d086528027/lib/RawSource.js#L12
        source.isBuffer ? source.source : source.source.toString("utf-8"));
    }
    if (!source.map) {
        return new webpack_sources_1.RawSource(source.source.toString("utf-8"));
    }
    return new webpack_sources_1.CompatSource({
        source() {
            return source.source.toString("utf-8");
        },
        map(_) {
            if (source.map) {
                return JSON.parse(source.map.toString("utf-8"));
            }
            return null;
        }
    });
}
exports.createSourceFromRaw = createSourceFromRaw;
function createRawFromSource(source) {
    const isBuffer = Buffer.isBuffer(source.source());
    if (source instanceof webpack_sources_1.RawSource) {
        return {
            source: source.buffer(),
            isRaw: true,
            isBuffer
        };
    }
    const buffer = source.buffer();
    const map = JSON.stringify(source.map({
        columns: true
    }));
    return {
        source: buffer,
        map: Buffer.from(map),
        isRaw: false,
        isBuffer
    };
}
exports.createRawFromSource = createRawFromSource;
//# sourceMappingURL=createSource.js.map